﻿using System;
using System.Windows.Media;
using System.Windows.Media.Media3D;
using System.Windows.Threading;
using Microsoft.Devices;
using Microsoft.Phone.Controls;
using SLARToolKit;

namespace ARDemo2 {
    public partial class MainPage : PhoneApplicationPage {
        DispatcherTimer dt;
        PhotoCamera cam;
        bool isInitialized;
        GenericMarkerDetector arDetector;
        int[] buffer;
        
        public MainPage() {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e) {
            cam = new PhotoCamera();
            cam.Initialized += PhotoCameraInitialized;
            isInitialized = false;
            viewfinderBrush.SetSource(cam);

            dt = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds(50) };
            dt.Tick += (sender, e1) => Detect();
            dt.Start();

            base.OnNavigatedTo(e);
        }

        void PhotoCameraInitialized(object sender, CameraOperationCompletedEventArgs e) {
            arDetector = new SLARToolKit.GenericMarkerDetector();
            var marker = Marker.LoadFromResource("data/Marker_SLAR_16x16segments_80width.pat", 16, 16, 80);           
            arDetector.Initialize((int)cam.PreviewResolution.Width, (int)cam.PreviewResolution.Height, 1, 4000, marker);
            isInitialized = true;
        }

        void Detect() {
            if (!isInitialized) return;
            var pixelWidth = (int)cam.PreviewResolution.Width;
            var pixelHeight = (int)cam.PreviewResolution.Height;
            if (buffer == null || buffer.Length != pixelWidth * pixelHeight) buffer = new int[pixelWidth * pixelHeight];
            
            cam.GetPreviewBufferArgb32(buffer);
            var reader = new camReader(buffer, pixelWidth, pixelHeight);

            var dr = arDetector.DetectAllMarkers(reader);
            if (dr.HasResults) {
                var centerAtOrigin = Matrix3DFactory.CreateTranslation(-250, -250, 0);
                var scale = Matrix3DFactory.CreateScale(0.5, -0.5, 0.5);
                var world = centerAtOrigin * scale * dr[0].Transformation;
                var viewport = Matrix3DFactory.CreateViewportTransformation(pixelWidth, pixelHeight);
                var m = Matrix3DFactory.CreateViewportProjection(world, Matrix3D.Identity, arDetector.Projection, viewport);
                var matrix3DProjection = new Matrix3DProjection { ProjectionMatrix = m };
                Img.Projection = matrix3DProjection;
                Img.Visibility = System.Windows.Visibility.Visible;
            } else
                Img.Visibility = System.Windows.Visibility.Collapsed;
        }

        class camReader : IXrgbReader {
            int[] _b;
            public camReader(int[] buffer, int width, int height) {
                _b = buffer;
                Width = width;
                Height = height;
            }
            
            public int[] GetAllPixels() {
                return _b;
            }

            public int GetPixel(int x, int y) {
                throw new NotImplementedException();
            }

            public int[] GetPixels(int[] x, int[] y) {
                throw new NotImplementedException();
            }

            public int Height { get; set; }
            public int Width { get; set; }
        }
    }
}