﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using Microsoft.Phone.Shell;

using Microsoft.FaceSdk.ImageHelper;
using Microsoft.FaceSdk.ImageHelper.IO;
using Microsoft.FaceSdk.Cartoon;
using Microsoft.FaceSdk.Common;
using Microsoft.FaceSdk.Image;
using Microsoft.FaceSdk.Cartoon.Framework;
using Microsoft.FaceSdk.Cartoon.Interfaces;
using Microsoft.FaceSdk.Cartoon.Filters;
using Microsoft.FaceSdk.Cartoon.Geometry;

using SdkRect = Microsoft.FaceSdk.Common.Rectangle;

namespace TestCartoon.Wp7
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);
        }

        private WriteableBitmap CreateCartoon(BitmapSource sourceImage)
        {
            // Size for the sketch
            int imageWidth = 512;
            int imageHeight = 512;
            Random rng = new Random(DateTime.Now.Millisecond);

            // Create engine
            CtnEngine engine = CtnEngine.Create();
            CtnSys sys = CtnEngine.CtnSystem;

            // You can retrieve Id(s) of all supported styles from the configuration
            // as the following code shows:
            //var filters = (from fc in sys.Filters
            //               where fc.Id == CtnGuid.GUID_CATEGORY_SKETCHGENERATION
            //               from f in fc.Filters
            //               select f).ToArray();
            //int generationId = rng.Next(0, filters.Length);
            //Guid generatorId = filters[generationId].Id;

            // In this example, we choose a fixed style according to the person in sample image:
            Guid generatorId = CtnGuid.GUID_GENERATOR_ASIAN_FEMALE_VECTOR;

            // Since internally the generator creator will use WriteableBitmap which is an UIElement object,
            // DO call this method in UI thread to avoid the issue of accessing UI element across threads!
            ISketchGeneration generator = engine.GetSketchGenerator(generatorId);

            // Create sketch
            ICtnSketch sketch = CtnEngine.CreateSketch();

            // Create filters
            IFaceDetection detector = engine.GetFaceDetector();
            IFaceAlignment alignmentor = engine.GetFaceAlignmentor();

            // Load image
            var img = ImageConverter.SystemToSdk(new WriteableBitmap(sourceImage));
            sketch.Image = img;

            var rect = detector.Detect(sketch);
            detector.FinalizeRect(sketch, rect);

            alignmentor.LoadRawSketch(sketch);
            sketch = alignmentor.GetSketch();

            SdkRect rcBound = new SdkRect(0, imageWidth, 0, imageHeight);
            generator.Generate(sketch, rcBound);

            // Apply exaggeration
            IFaceExaggeration exaggerator = engine.GetFaceExaggerator();
            exaggerator.Exaggerate(sketch, ExaggerationType.AutoAddConstr, 0.0f);

            // Sketch generation finished.

            // Apply template (non animation type)
            var templateFilters = engine.GetCartoonTemplates(generatorId);
            foreach (var tf in templateFilters)
            {
                if (tf.Count == 0) continue;

                // Skip animation templates
                if (tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_ANIMATEBODYCLOTH ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_ANIMATEBODYCLOTH_P ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_ANIMATEEXPRESSION ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_MULTIPART_ANIMATION)
                    continue;

                // Randomly apply certain templates
                if (tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_EXPRESSION ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_EXPRESSION_P ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_PET ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_PET_P ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_GLASSES ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_GLASS_P ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_BACKGROUND ||
                    tf.TemplateId == CtnGuid.GUID_TEMPLATETYPE_BACKGROUND_P)
                {
                    if (rng.Next(0, 2) == 0) continue;
                }

                int index = rng.Next(0, tf.Count);
                tf.Apply(sketch, index);
            }

            // Render sketch
            float ratio, tx, ty;
            sketch.FitToRect(SdkRect.FromLTWH(0, 0, imageWidth, imageHeight), out ratio, out tx, out ty);
            var sketchImage = sketch.Render(imageWidth, imageHeight);

            var bmp = ImageConverter.SdkToSystem(sketchImage);
            return bmp;
        }

        private void OnRun(object sender, RoutedEventArgs e)
        {
            this.runButton.Visibility = Visibility.Collapsed;
            this.message.Visibility = Visibility.Visible;

            var image = this.canvas.Source as BitmapImage;
            Debug.Assert(image != null);

            Dispatcher.BeginInvoke(() => {
                try
                {
                    Stopwatch sw = Stopwatch.StartNew();
                    this.canvas.Source = this.CreateCartoon(image);
                    sw.Stop();

                    this.infoBlk.Text = string.Format("Generation time: {0} ms", sw.ElapsedMilliseconds);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message + "\n" + ex.StackTrace);
                }
                finally
                {
                    this.resetButton.Visibility = System.Windows.Visibility.Visible;
                    this.message.Visibility = System.Windows.Visibility.Collapsed;
                }
            });
        }

        private void OnReset(object sender, RoutedEventArgs e)
        {
            BitmapImage image = new BitmapImage(new Uri("/Assets/sample.jpg", UriKind.Relative));
            this.canvas.Source = image;
            this.runButton.Visibility = Visibility.Visible;
            this.resetButton.Visibility = System.Windows.Visibility.Collapsed;
            this.infoBlk.Text = string.Empty;
        }
    }
}