﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.ComponentModel;

using Microsoft.FaceSdk.ImageHelper;

namespace FaceAlignmentDemo.Wp7
{
    public partial class MainPage : PhoneApplicationPage
    {
        private SdkHelper helper;
        private WriteableBitmap bitmap;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            this.helper = this.Resources["sdkhelper"] as SdkHelper;
            Debug.Assert(helper != null);

            this.Loaded += (sender, e) =>
            {
                var si = Application.GetResourceStream(new Uri("Assets/sample.jpg", UriKind.Relative));
                BitmapImage source = new BitmapImage();
                source.CreateOptions = BitmapCreateOptions.None;
                source.SetSource(si.Stream);

                this.bitmap = new WriteableBitmap(source);
                this.img.Source = this.bitmap;
            };

            this.detectBtn.IsEnabled = false;
            this.resetBtn.IsEnabled = false;

            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += (sender, e) =>
            {
                // The initialization stage might take some time.
                // Do this in a background thread to avoid blocking the UI thread.
                this.helper.Initialize();

                Dispatcher.BeginInvoke(() => 
                {
                    this.detectBtn.IsEnabled = true;
                    this.resetBtn.IsEnabled = true;
                    this.StatusTxt.Text = "Ready";
                });
            };

            worker.RunWorkerAsync();
        }

        private void OnDetectButtonClicked(object sender, RoutedEventArgs e)
        {
            var sdkImg = ImageConverter.SystemToSdk(this.bitmap);
            var result = this.helper.Detect(sdkImg);
            this.img.Source = ImageConverter.SdkToSystem(result);
        }

        private void OnResetButtonClicked(object sender, RoutedEventArgs e)
        {
            this.img.Source = this.bitmap;
        }
    }
}