﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using Microsoft.FaceSdk.Common;
using Microsoft.FaceSdk.Image;
using Microsoft.FaceSdk.ImageHelper;
using Microsoft.Phone.Controls;
using UIFramework.Common;
using UIFramework.Controls;
using UIFramework.Pages;
using UIFramework.Photo;
using FaceParty.Localization;
using System.Text;

namespace FaceParty
{
    public partial class EditPage : PhoneApplicationPage
    {
        public EditPage()
        {
            InitializeComponent();
            
            //set the facedetecting animation 
            this.faceDetectingAnimation.MobileBrushResource = new SolidColorBrush(Constant.MobileAnimationColor);
            this.faceDetectingAnimation.DetectingReminder = Localization.Strings.DetectingState;
            this.faceDetectingAnimation.FailedReminder = Localization.Strings.FailedState;
            this.faceDetectingAnimation.ShakePhoneReminder = Localization.Strings.SuccessState;

            //mark the detection failed, to handle the share event.
            this.IsDetectionFailed = false;
            
            this.LayoutRoot.DataContext = this;

            this.LayoutUpdated += (sender, e) =>
            {
                var source = this.imgViewer.Source as BitmapImage;
                if (source != null) source.CreateOptions = BitmapCreateOptions.None;

                this.transform.Strech = this.imgViewer.Stretch;
                this.transform.ViewSize = this.imgViewer.DesiredSize;

                if (source != null)
                {
                    this.transform.SourceSize = new System.Windows.Size(source.PixelWidth, source.PixelHeight);
                }

                if (this.imgViewer.DesiredSize.Width != 0 && this.imgViewer.DesiredSize.Height != 0)
                    UpdateMaskLayout();
            };
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (e.NavigationMode == NavigationMode.New)
            {
                // collect active photo stream from photo manager;
                // which is input from photo chooser;
                BitmapImage currentImage = new BitmapImage();
                currentImage.CreateOptions = BitmapCreateOptions.None;
                currentImage.SetSource(persistence.PhotoManager.ActivePhoto.OriginalStream);
                this.imgViewer.Source = currentImage;

                // Calculate the transform
                this.transform.SourceSize = new System.Windows.Size(currentImage.PixelWidth, currentImage.PixelHeight);

                // Get raw image data
                this.rawBitmap = new WriteableBitmap(currentImage);
                this.rawImage = ImageConverter.SystemToSdk(rawBitmap);

                // Try to parse the meta data stream
                this.currentFaces = GetMetaData(persistence.PhotoManager.ActivePhoto);
                if (this.currentFaces == null || this.currentFaces == null)
                {
                    // If not available, consider the input as a new project and detect the face(s) in it
                    Dispatcher.BeginInvoke(() => { this.WorkThread(rawImage); });
                }
                else
                {
                    foreach (var f in this.currentFaces)
                    {
                        // Clear state
                        var mask = CreateMask(f);
                        this.maskCanvas.Children.Add(mask);
                    }
                }
            }

            base.OnNavigatedTo(e);
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            if (e.Content is PortraitSharePage)
            {
                // set the properties of share page
                var sharePage = e.Content as PortraitSharePage;
                sharePage.ShareImageSource = shareBitmap;
            }

            PhotoManager photoManager = persistence.PhotoManager;
            photoManager.EditPhoto(rawBitmap);
            SaveMetaData(photoManager.ActivePhoto, this.currentFaces);

            if (e.Content is MainPage)
            {
                // navigate to main page, it's end of editing;
                photoManager.EndEditPhoto();
            }
            else
            {
                // not end of editing, navigate to external or other pages, such as shareing.
                photoManager.SaveInEditing();
            }

            base.OnNavigatedFrom(e);
        }

        private void OnResetButtonClicked(object sender, RoutedEventArgs e)
        {
            Reset();
        }

        private void OnSaveButtonClicked(object sender, RoutedEventArgs e)
        {
            WriteableBitmap sourceImage = new WriteableBitmap(this.imgViewer.Source as BitmapSource);
            var wb = ImageHelper.CreateWaterMark(sourceImage);

            string filename = string.Format(
                     CultureInfo.InvariantCulture,
                     "{0}_{1}{2}",
                     Constant.ImageFileName,
                     DateTime.UtcNow.ToString(Constant.DateTimeFormat),
                     Constant.ImageFileSuffix
                     );

            if (ImageHelper.SaveImage(wb, filename))
            {
                this.mmsgbox.Text = FaceParty.Localization.Strings.success_save_info;
                this.mmsgbox.Show();
            }
            else
            {
                MessageBox.Show(FaceParty.Localization.Strings.fail_to_save_image_info, FaceParty.Localization.Strings.fail_to_save_image, MessageBoxButton.OK);
            }
        }

        // bitmap holder for sharing;
        WriteableBitmap shareBitmap = null;        
        private void OnShareButtonClicked(object sender, RoutedEventArgs e)
        {
            // Don't put CreateWaterMark in OnNavigatedFrom, in which bitmap will not be rendered properly;
            shareBitmap = ImageHelper.CreateWaterMark(this.rawBitmap);

            StringBuilder uriBuild = new StringBuilder("/UIFramework;component/Pages/PortraitSharePage.xaml");
            uriBuild.AppendFormat("?FacebookAppId={0}", Constant.FacebookAppId);
            uriBuild.AppendFormat("&FacebookAppSecret={0}", Constant.FacebookAppSecret);

            uriBuild.AppendFormat("&TwitterAppCallBack={0}", Constant.TwitterAppCallBack);
            uriBuild.AppendFormat("&TwitterAppSecret={0}", Constant.TwitterAppSecret);
            uriBuild.AppendFormat("&TwitterAppKey={0}", Constant.TwipicAppKey);

            uriBuild.AppendFormat("&SinaWeiboAppId={0}", Constant.SinaWeiboAppId);
            uriBuild.AppendFormat("&SinaWeiboAppSecret={0}", Constant.SinaWeiboAppSecret);
            uriBuild.AppendFormat("&SinaWeiboAppCallBack={0}", Constant.SinaWeiboAppCallBack);

            uriBuild.AppendFormat("&WindowsLiveAppId={0}", Constant.WindowsLiveAppId);
            uriBuild.AppendFormat("&WinsowsLiveAppSecret={0}", Constant.WinsowsLiveAppSecret);
            NavigationService.Navigate(new Uri(uriBuild.ToString(), UriKind.Relative));
        }

        private void OnEditButtonClicked(object sender, RoutedEventArgs e)
        {
        }

        /// <summary>
        /// Remove mask
        /// </summary>
        private void OnRemoveMask(object sender, RoutedEventArgs e)
        {
            if (currentFaces == null)
            {
                return;
            }

            // enable reset button after remove one mask;
            resetButton.IsEnabled = true;

            // the sender is the mask control thus we can safely cast it to UIElement and remove it from
            // the panel
            MaskControl mask = sender as MaskControl;
            mask.Visibility = System.Windows.Visibility.Collapsed;
        }
    }
}
