﻿/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Xna.Framework;

// Directives
using Microsoft.Devices;
using System.Windows.Media.Imaging;
using System.Threading;

using Microsoft.FaceSdk.Tracker;
using Microsoft.FaceSdk.Image;
using Microsoft.FaceSdk.ImageHelper;

namespace FaceTrackDemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Variables
        PhotoCamera cam = new PhotoCamera();
        private static ManualResetEvent pauseFramesEvent = new ManualResetEvent(true);
        private Thread ARGBFramesThread;
        private bool pumpARGBFrames;
        private MOTFaceTracker tracker = new MOTFaceTracker();

        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        //Code for camera initialization event, and setting the source for the viewfinder
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {

            // Check to see if the camera is available on the device.
            if ((PhotoCamera.IsCameraTypeSupported(CameraType.Primary) == true) ||
                 (PhotoCamera.IsCameraTypeSupported(CameraType.FrontFacing) == true))
            {
                // Initialize the default camera.
                cam = new Microsoft.Devices.PhotoCamera();

                //Event is fired when the PhotoCamera object has been initialized
                cam.Initialized += cam_Initialized;

                //Set the VideoBrush source to the camera
                viewfinderBrush.SetSource(cam);
            }
            else
            {
                // The camera is not supported on the device.
                this.Dispatcher.BeginInvoke(delegate()
                {
                    // Write message.
                    txtDebug.Text = "A Camera is not available on this device.";
                });

                // Disable UI.
                TrackOnButton.IsEnabled = false;
                TrackOffButton.IsEnabled = false;
            }
        }

        protected override void OnNavigatingFrom(System.Windows.Navigation.NavigatingCancelEventArgs e)
        {
            if (cam != null)
            {
                // Dispose of the camera to minimize power consumption and to expedite shutdown.
                cam.Dispose();

                // Release memory, ensure garbage collection.
                cam.Initialized -= cam_Initialized;
            }
        }

        //Update UI if initialization succeeds
        void cam_Initialized(object sender, Microsoft.Devices.CameraOperationCompletedEventArgs e)
        {
            if (e.Succeeded)
            {
                this.Dispatcher.BeginInvoke(delegate()
                {
                    txtDebug.Text = "Camera initialized";
                });
            }
        }

        // Frame pump
        void PumpARGBFrames()
        {
            // Create capture buffer.
            var input = new Microsoft.FaceSdk.Image.ImageGray((int)cam.PreviewResolution.Width, (int)cam.PreviewResolution.Height);

            PhotoCamera phCam = (PhotoCamera)cam;

            while (pumpARGBFrames)
            {
                pauseFramesEvent.WaitOne();

                phCam.GetPreviewBufferY(input.Buffer);

                Stopwatch sw = Stopwatch.StartNew();
                var rc = this.tracker.Track(input);
                sw.Stop();

                pauseFramesEvent.Reset();
                Deployment.Current.Dispatcher.BeginInvoke(delegate()
                {
                    txtDebug.Text = string.Format("track time: {0} ms, {1}", sw.ElapsedMilliseconds, !rc.IsEmpty ? "Yes" : "No");

                    this.faceRect.Margin = new Thickness(rc.Left, rc.Top, 0, 0);
                    this.faceRect.Width = rc.Width;
                    this.faceRect.Height = rc.Height;
                    this.faceRect.Visibility = System.Windows.Visibility.Visible;

                    pauseFramesEvent.Set();
                });
            }

        }

        // Start frame pump.
        private void TrackOn_Clicked(object sender, RoutedEventArgs e)
        {
            if (ARGBFramesThread != null && ARGBFramesThread.IsAlive)
            {
                MessageBox.Show("Please stop tracking first!");
            }
            else
            {
                pumpARGBFrames = true;
                ARGBFramesThread = new System.Threading.Thread(PumpARGBFrames);

                // Start pump.
                ARGBFramesThread.Start();
                this.Dispatcher.BeginInvoke(delegate()
                {
                    txtDebug.Text = "Tracking ...";
                });
            }
        }

        // Stop frame pump.
        private void TrackOff_Clicked(object sender, RoutedEventArgs e)
        {
            pumpARGBFrames = false;
            this.Dispatcher.BeginInvoke(delegate()
            {
                txtDebug.Text = "";
            });
        }
    }
}